@@ -61,7 +61,7 @@ class Lambda(nn.Module):
 
 class GlobalPool(nn.Module):
     """computes values reduced over all spatial locations (& group elements) in the mask"""
-    def __init__(self,mean=False):
+    def __init__(self, mean = False):
         super().__init__()
         self.mean = mean
 
@@ -73,10 +73,12 @@ class GlobalPool(nn.Module):
 
         masked_vals = vals.masked_fill_(~mask[..., None], 0.)
         summed = masked_vals.sum(dim = 1)
-        if self.mean:
-            count = mask.sum(-1).unsqueeze(-1)
-            summed /= count
-        return summed
+
+        if not self.mean:
+            return summed
+
+        count = mask.sum(-1).unsqueeze(-1)
+        return summed / count
 
 # subsampling code
 
@@ -85,35 +87,38 @@ def FPSindices(dists, frac, mask):
         outputs: chosen_indices (bs,m) """
     m = int(round(frac * dists.shape[1]))
     bs, n, device = *dists.shape[:2], dists.device
+    dd_kwargs = {'device': device, 'dtype': torch.long}
+    B = torch.arange(bs, **dd_kwargs)
 
-    chosen_indices = torch.zeros(bs, m, dtype=torch.long, device=device)
-    distances = torch.ones(bs, n,device=device) * 1e8
-    a = torch.randint(0, n, (bs,), dtype=torch.long, device=device) #choose random start
-    idx = a % mask.sum(-1) + torch.cat([torch.zeros(1, device=device).long(), torch.cumsum(mask.sum(-1),dim=0)[:-1]], dim=0)
+    chosen_indices = torch.zeros(bs, m, **dd_kwargs)
+    distances = torch.ones(bs, n, device=device) * 1e8
+    a = torch.randint(0, n, (bs,), **dd_kwargs)            # choose random start
+    idx = a % mask.sum(-1) + torch.cat([torch.zeros(1, **dd_kwargs), torch.cumsum(mask.sum(-1), dim=0)[:-1]], dim=0)
     farthest = torch.where(mask)[1][idx]
 
     for i in range(m):
-        chosen_indices[:, i] = farthest                  # add point that is farthest to chosen
-        dist = batched_index_select(dists, farthest, dim = 1).masked_fill(mask, -100) # (bs,n) compute distance from new point to all others
-        closer = dist < distances                        # if dist from new point is smaller than chosen points so far
-        distances[closer] = dist[closer]                 # update the chosen set's distance to all other points
-        farthest = torch.max(distances, -1)[1]           # select the point that is farthest from the set
+        chosen_indices[:, i] = farthest                    # add point that is farthest to chosen
+        dist = dists[B, farthest].masked_fill(~mask, -100) # (bs,n) compute distance from new point to all others
+        closer = dist < distances                          # if dist from new point is smaller than chosen points so far
+        distances[closer] = dist[closer]                   # update the chosen set's distance to all other points
+        farthest = torch.max(distances, -1)[1]             # select the point that is farthest from the set
 
     return chosen_indices
 
 
 class FPSsubsample(nn.Module):
-    def __init__(self, ds_frac, cache=False, group=None):
+    def __init__(self, ds_frac, cache = False, group = None):
         super().__init__()
         self.ds_frac = ds_frac
         self.cache = cache
         self.cached_indices = None
-        self.group = group
+        self.group = default(group, SE3())
 
     def get_query_indices(self, abq_pairs, mask):
         if self.cache and exists(self.cached_indices):
             return self.cached_indices
 
+        dist = self.group.distance if self.group else lambda ab: ab.norm(dim=-1)
         value = FPSindices(dist(abq_pairs), self.ds_frac, mask).detach()
 
         if self.cache:
@@ -125,10 +130,8 @@ class FPSsubsample(nn.Module):
         abq_pairs,vals,mask = inp
         device = vals.device
 
-        dist = self.group.distance if self.group else lambda ab: ab.norm(dim=-1)
-
-        if self.ds_frac!=1:
-            query_idx = get_query_indices(abq_pairs, mask)
+        if self.ds_frac != 1:
+            query_idx = self.get_query_indices(abq_pairs, mask)
 
             B = torch.arange(query_idx.shape[0], device = device).long()[:,None]
             subsampled_abq_pairs = abq_pairs[B, query_idx][B, :, query_idx]
@@ -156,48 +159,40 @@ class FPSsubsample(nn.Module):
 class LieSelfAttention(nn.Module):
     def __init__(
         self,
-        chin,
+        dim,
+        group = None,
         mc_samples = 32,
-        xyz_dim = 3,
         ds_frac = 1,
         loc_attn = False,
-        knn_channels = None,
-        mean = False,
-        group = SE3,
-        fill = 1/3,
-        cache = False,
-        knn = False,
+        fill = 1 / 3,
         dim_head = 64,
         heads = 8,
         attend_self = True,
-        **kwargs
+        cache = False
     ):
         super().__init__()
-        self.chin = chin # input channels
-        self.cmco_ci = 16 # a hyperparameter controlling size and bottleneck compute cost of weightnet
-        self.xyz_dim = xyz_dim # dimension of the space on which convolution operates
-        self.knn_channels = knn_channels # number of xyz dims on which to compute knn
+        self.dim = dim
+
         self.mc_samples = mc_samples # number of samples to use to estimate convolution
-
-        self.mean=mean  # Whether or not to divide by the number of mc_samples
-
-        self.group = group # Equivariance group for LieConv
+        self.group = default(group, SE3()) # Equivariance group for LieConv
         self.register_buffer('r',torch.tensor(2.)) # Internal variable for local_neighborhood radius, set by fill
-        self.fill_frac = min(fill,1.) # Average Fraction of the input which enters into local_neighborhood, determines r
-        self.knn = knn            # Whether or not to use the k nearest points instead of random samples for conv estimator
+        self.fill_frac = min(fill, 1.) # Average Fraction of the input which enters into local_neighborhood, determines r
 
-        self.subsample = FPSsubsample(ds_frac, cache=cache, group=self.group)
+        self.subsample = FPSsubsample(ds_frac, cache = cache, group = self.group)
         self.coeff = .5  # Internal coefficient used for updating r
+
         self.fill_frac_ema = fill # Keeps track of average fill frac, used for logging only
 
+        # attention related parameters
+
         inner_dim = dim_head * heads
         self.heads = heads
         self.attend_self = attend_self
 
-        self.to_q = nn.Linear(chin, inner_dim, bias = False)
-        self.to_k = nn.Linear(chin, inner_dim, bias = False)
-        self.to_v = nn.Linear(chin, inner_dim, bias = False)
-        self.to_out = nn.Linear(inner_dim, chin)
+        self.to_q = nn.Linear(dim, inner_dim, bias = False)
+        self.to_k = nn.Linear(dim, inner_dim, bias = False)
+        self.to_v = nn.Linear(dim, inner_dim, bias = False)
+        self.to_out = nn.Linear(inner_dim, dim)
 
         self.loc_attn_mlp = nn.Sequential(
             nn.Linear(self.group.lie_dim, self.group.lie_dim * 4),
@@ -213,7 +208,7 @@ class LieSelfAttention(nn.Module):
         pairs_abq, inp_vals, mask = inp
         device = inp_vals.device
 
-        if query_indices is not None:
+        if exists(query_indices):
             abq_at_query = batched_index_select(pairs_abq, query_indices, dim = 1)
             mask_at_query = batched_index_select(mask, query_indices, dim = 1)
         else:
@@ -224,21 +219,17 @@ class LieSelfAttention(nn.Module):
 
         vals_at_query = inp_vals
         dists = self.group.distance(abq_at_query) #(bs,m,n,d) -> (bs,m,n)
-        dists = dists.masked_fill(mask[:,None,:].expand(*dists.shape), 1e8)
+        mask_value = torch.finfo(dists.dtype).max
+        dists = dists.masked_fill(mask[:,None,:], mask_value)
 
-        k = min(self.mc_samples,inp_vals.shape[1])
+        k = min(self.mc_samples, inp_vals.shape[1])
 
-        # Determine ids (and mask) for points sampled within neighborhood (A4)
-        if self.knn: # NBHD: KNN
-            nbhd_idx = torch.topk(dists,k,dim=-1,largest=False,sorted=False)[1] #(bs,m,nbhd)
-            valid_within_ball = (nbhd_idx > -1) & mask[:,None,:] & mask_at_query
-            assert not torch.any(nbhd_idx > dists.shape[-1]), f"error with topk, nbhd{k} nans|inf{torch.any(torch.isnan(dists)|torch.isinf(dists))}"
-        else: # NBHD: Sampled Distance Ball
-            bs, m, n = dists.shape
-            within_ball = (dists < self.r) & mask[:,None,:] & mask_at_query # (bs,m,n)
-            noise = torch.zeros((bs, m, n), device = device).uniform_(0, 1)
-            valid_within_ball, nbhd_idx = torch.topk(within_ball + noise, k, dim=-1, sorted=False)
-            valid_within_ball = (valid_within_ball > 1)
+        # NBHD: Sampled Distance Ball
+        bs, m, n = dists.shape
+        within_ball = (dists < self.r) & mask[:,None,:] & mask_at_query # (bs,m,n)
+        noise = torch.zeros((bs, m, n), device = device).uniform_(0, 1)
+        valid_within_ball, nbhd_idx = torch.topk(within_ball + noise, k, dim=-1, sorted=False)
+        valid_within_ball = (valid_within_ball > 1)
 
         # Retrieve abq_pairs, values, and mask at the nbhd locations
 
@@ -246,7 +237,7 @@ class LieSelfAttention(nn.Module):
         nbhd_vals = batched_index_select(vals_at_query, nbhd_idx, dim = 1)
         nbhd_mask = batched_index_select(mask, nbhd_idx, dim = 1)
 
-        if self.training and not self.knn: # update ball radius to match fraction fill_frac inside
+        if self.training: # update ball radius to match fraction fill_frac inside
             navg = (within_ball.float()).sum(-1).sum() / mask_at_query.sum()
             avg_fill = (navg / mask.sum(-1).float().mean()).cpu().item()
             self.r +=  self.coeff * (self.fill_frac - avg_fill)
@@ -332,16 +323,16 @@ class FeedForward(nn.Module):
 
 class LieTransformer(nn.Module):
     """
-        [Fill] specifies the fraction of the input which is included in local neighborhood.
-                (can be array to specify a different value for each layer)
-        [nbhd] number of samples to use for Monte Carlo estimation (p)
-        [chin] number of input channels: 1 for MNIST, 3 for RGB images, other for non images
-        [ds_frac] total downsampling to perform throughout the layers of the net. In (0,1)
-        [num_layers] number of BottleNeck Block layers in the network
-        [k] channel width for the network. Can be int (same for all) or array to specify individually.
-        [liftsamples] number of samples to use in lifting. 1 for all groups with trivial stabilizer. Otherwise 2+
-        [Group] Chosen group to be equivariant to.
-        """
+    [Fill] specifies the fraction of the input which is included in local neighborhood.
+            (can be array to specify a different value for each layer)
+    [nbhd] number of samples to use for Monte Carlo estimation (p)
+    [dim] number of input channels: 1 for MNIST, 3 for RGB images, other for non images
+    [ds_frac] total downsampling to perform throughout the layers of the net. In (0,1)
+    [num_layers] number of BottleNeck Block layers in the network
+    [k] channel width for the network. Can be int (same for all) or array to specify individually.
+    [liftsamples] number of samples to use in lifting. 1 for all groups with trivial stabilizer. Otherwise 2+
+    [Group] Chosen group to be equivariant to.
+    """
     def __init__(
         self,
         dim,
@@ -356,15 +347,14 @@ class LieTransformer(nn.Module):
         mean = True,
         per_point = True,
         liftsamples = 4,
-        fill = 1/4,
-        knn = False,
+        fill = 1 / 4,
         cache = False,
         **kwargs
     ):
         super().__init__()
         dim_out = default(dim_out, dim)
 
-        if isinstance(fill,(float,int)):
+        if isinstance(fill, (float, int)):
             fill = [fill] * depth
 
         group = SE3()
@@ -372,7 +362,7 @@ class LieTransformer(nn.Module):
         self.liftsamples = liftsamples
 
         block = lambda dim, fill: nn.Sequential(
-            LieSelfAttentionWrapper(dim, attn = partial(LieSelfAttention, dim, heads = heads, dim_head = dim_head, loc_attn = loc_attn, mc_samples=nbhd, ds_frac=ds_frac, mean=mean, group=group,fill=fill,cache=cache,knn=knn,**kwargs)),
+            LieSelfAttentionWrapper(dim, attn = partial(LieSelfAttention, dim, heads = heads, dim_head = dim_head, loc_attn = loc_attn, mc_samples=nbhd, ds_frac=ds_frac, group=group, fill=fill, cache=cache,**kwargs)),
             FeedForward(dim)
         )
 
